@@ -8,7 +8,6 @@ glm.mi <- function (formula, mi.object, family = gaussian, ... )
     result <- vector( "list", m )
     names( result ) <- as.character(paste( "Imputation", seq( m ), sep = "" ))
     mi.data <- mi.completed(mi.object)
-    mi.data <- mi.postprocess(mi.data)
   
     for ( i in 1:m ) {
       result[[i]] <- glm( formula, family = family, 
@@ -20,18 +19,10 @@ glm.mi <- function (formula, mi.object, family = gaussian, ... )
       coef[[j]]<- lapply( result, summary )[[ j ]]$coef[ ,1]
       se[[j]]  <- lapply( result, summary )[[ j ]]$coef[ ,2]
     }
-    W         <- colSums( do.call( rbind, se )^2 ) / m
-    Bhat      <- colSums( do.call( rbind, coef ) ) / m
-    Bhat_rep  <- t( matrix( rep( Bhat, m ), length( Bhat ), m ) )
-    B         <- colSums( do.call( rbind, coef ) - Bhat_rep ) ^ 2 / ( m - 1 )
-    
-    pooled <- list( coefficients = NULL, se = NULL )
-    pooled$coefficients <- Bhat
-    pooled$se <- sqrt( W + ( 1 + 1 / m ) * B )
-    
-    mi.glm.object <- new("mi.glm",
+    pooled <- mi.pooled(coef, se, m)
+    mi.pooled.object <- new("mi.pooled",
                           call = call, 
                           mi.pooled = pooled,
-                          mi.fit = result )
-    return( mi.glm.object )
+                          mi.fit = result)
+    return( mi.pooled.object )
 }
